package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.CentralizedRequestDTO;
import digital.laboratory.platform.reagent.entity.CentralizedRequest;
import digital.laboratory.platform.reagent.entity.DetailsOfCentralized;
import digital.laboratory.platform.reagent.mapper.CentralizedRequestMapper;
import digital.laboratory.platform.reagent.service.CentralizedRequestService;
import digital.laboratory.platform.reagent.service.DetailsOfCentralizedService;
import digital.laboratory.platform.reagent.vo.CentralizedRequestVO;
import digital.laboratory.platform.reagent.vo.DetailsOfCentralizedVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (集中采购申请)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (集中采购申请) 服务实现类
 */
@Service
public class CentralizedRequestServiceImpl extends ServiceImpl<CentralizedRequestMapper, CentralizedRequest> implements CentralizedRequestService {

    @Autowired
    private CentralizedRequestService centralizedRequestService;

    @Autowired
    private DetailsOfCentralizedService detailsOfCentralizedService;

    @Override
    public List<DetailsOfCentralized> saveRequestById(CentralizedRequest centralizedRequest, List<CentralizedRequestDTO> centralizedRequestDTOList, DLPUser dlpUser) {

        centralizedRequest.setApplicantId(dlpUser.getId());

        centralizedRequest.setCentralizedRequestId(IdWorker.get32UUID().toUpperCase());
        //获取申请时间
        centralizedRequest.setDateOfApplication(LocalDateTime.now());
        //未提交
        centralizedRequest.setStatus(0);

        List<DetailsOfCentralized> detailsOfCentralizedList = new ArrayList<>();
        //将DTO赋值给明细表
        for (CentralizedRequestDTO centralizedRequestDto : centralizedRequestDTOList) {

            DetailsOfCentralized detailsOfCentralized = new DetailsOfCentralized();

            BeanUtils.copyProperties(centralizedRequestDto, detailsOfCentralized);

            detailsOfCentralized.setDetailsOfCentralizedId(IdWorker.get32UUID().toUpperCase());

            detailsOfCentralized.setCentralizedRequestId(centralizedRequest.getCentralizedRequestId());

            detailsOfCentralizedList.add(detailsOfCentralized);
        }
        return detailsOfCentralizedList;
    }


    @Override
    public CentralizedRequest commit(String centralizedRequestId) {

        CentralizedRequest centralizedRequest = centralizedRequestService.getById(centralizedRequestId);

        LambdaQueryWrapper<DetailsOfCentralized> detailsOfCentralizedLambdaQueryWrapper = new LambdaQueryWrapper<>();

        detailsOfCentralizedLambdaQueryWrapper.eq(DetailsOfCentralized::getCentralizedRequestId, centralizedRequestId);

        List<DetailsOfCentralized> list = detailsOfCentralizedService.list(detailsOfCentralizedLambdaQueryWrapper);
        //语义检查
        for (DetailsOfCentralized detailsOfCentralized : list) {

            if (detailsOfCentralized.getPurchaseCatalogueNumber() != null &
                    detailsOfCentralized.getQuantityPurchased() > 0 & detailsOfCentralized.getQuantityPurchased() < 999 &
                    detailsOfCentralized.getPurchaseCatalogueNumber() != null &
                    detailsOfCentralized.getReagentConsumableId() != null) {
            } else {
                return null;
            }
        }
        centralizedRequest.setStatus(1);

        return centralizedRequest;
    }

    @Override
    public DetailsOfCentralized editDetails(CentralizedRequestDTO centralizedRequestDto) {

        DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getById(centralizedRequestDto.getDetailsOfCentralizedId());

        BeanUtils.copyProperties(centralizedRequestDto, detailsOfCentralized);

        return detailsOfCentralized;

    }

    @Override
    public DetailsOfCentralized addDetails(CentralizedRequestDTO centralizedRequestDto) {

        DetailsOfCentralized detailsOfCentralized = new DetailsOfCentralized();

        detailsOfCentralized.setDetailsOfCentralizedId(IdWorker.get32UUID().toUpperCase());

        BeanUtils.copyProperties(centralizedRequestDto, detailsOfCentralized);

        return detailsOfCentralized;
    }

    @Override
    public List<DetailsOfCentralized> delRequestById(String centralizedRequestId) {

        LambdaQueryWrapper<DetailsOfCentralized> detailsOfCentralizedLambdaQueryWrapper = new LambdaQueryWrapper<>();

        detailsOfCentralizedLambdaQueryWrapper.eq(DetailsOfCentralized::getCentralizedRequestId, centralizedRequestId);

        List<DetailsOfCentralized> list = detailsOfCentralizedService.list(detailsOfCentralizedLambdaQueryWrapper);

        return list;

    }

    @Override
    public CentralizedRequest checkRequest(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        CentralizedRequest centralizedRequest = centralizedRequestService.getById(auditAndApproveDto.getUuId());


        if (centralizedRequest != null) {
            centralizedRequest.setAuditResult(auditAndApproveDto.getAuditResult());

            centralizedRequest.setAuditOpinion(auditAndApproveDto.getAuditOpinion());

            centralizedRequest.setAuditorId(dlpUser.getId());

            centralizedRequest.setAuditTime(LocalDateTime.now());

            centralizedRequest.setStatus(2);

            return centralizedRequest;
        } else {
            return null;
        }
    }

    @Override
    public IPage<CentralizedRequestVO> getCentralizedRequestVOPage(IPage<CentralizedRequest> page, QueryWrapper<CentralizedRequest> qw) {
        IPage<CentralizedRequestVO> r = baseMapper.getCentralizedRequestVOPage(page, qw);
        return r;
    }

    @Override
    public List<CentralizedRequestVO> getCentralizedRequestVOList(QueryWrapper<CentralizedRequest> qw) {
        List<CentralizedRequestVO> list = baseMapper.getCentralizedRequestVOList(qw);
        return list;
    }

    @Override
    public CentralizedRequestVO getCentralizedRequestVO(String centralizedRequestId) {

        CentralizedRequestVO centralizedRequestVO = baseMapper.getCentralizedRequestVO(centralizedRequestId);

        List<DetailsOfCentralizedVO> detailsOfCentralizedVOList = detailsOfCentralizedService.getDetailsOfCentralizedVOList(centralizedRequestId);

        centralizedRequestVO.setDetailsOfCentralizedVOList(detailsOfCentralizedVOList);

        return centralizedRequestVO;
    }
}



