package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.DecentralizedRequestDTO;
import digital.laboratory.platform.reagent.entity.DecentralizeDetails;
import digital.laboratory.platform.reagent.entity.DecentralizedRequest;
import digital.laboratory.platform.reagent.mapper.DecentralizedRequestMapper;
import digital.laboratory.platform.reagent.service.DecentralizeDetailsService;
import digital.laboratory.platform.reagent.service.DecentralizedRequestService;
import digital.laboratory.platform.reagent.vo.DecentralizeDetailsVO;
import digital.laboratory.platform.reagent.vo.DecentralizedRequestVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (分散采购申请)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (分散采购申请) 服务实现类
 */
@Service
public class DecentralizedRequestServiceImpl extends ServiceImpl<DecentralizedRequestMapper, DecentralizedRequest> implements DecentralizedRequestService {

    @Autowired
    private DecentralizedRequestService decentralizedRequestService;
    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Override//分散采购申请分页
    public IPage<DecentralizedRequestVO> getDecentralizedRequestVOPage
            (@Param("page") IPage<DecentralizedRequest> page, @Param(Constants.WRAPPER) QueryWrapper<DecentralizedRequest> qw) {

        IPage<DecentralizedRequestVO> decentralizedRequestVOPage = baseMapper.getDecentralizedRequestVOPage(page, qw);

        return decentralizedRequestVOPage;
    }


    @Override//分散采购申请列表
    public List<DecentralizedRequestVO> getDecentralizedRequestVOList(@Param(Constants.WRAPPER) QueryWrapper<DecentralizedRequest> qw) {

        List<DecentralizedRequestVO> decentralizedRequestVOList = baseMapper.getDecentralizedRequestVOList(qw);

        return decentralizedRequestVOList;
    }

    @Override//通过ID查询分散采购申请
    public DecentralizedRequestVO getRequestById(String decentralizedRequestId) {

        DecentralizedRequestVO decentralizedRequestVO = baseMapper.getDecentralizedRequest(decentralizedRequestId);

        List<DecentralizeDetailsVO> decentralizeDetailsVOList = decentralizeDetailsService.getDecentralizeDetailsVOList(decentralizedRequestId);

        decentralizedRequestVO.setDecentralizeDetailsVOList(decentralizeDetailsVOList);

        return decentralizedRequestVO;
    }

    @Override//创建分散采购申请
    public List<DecentralizeDetails> addRequest(List<DecentralizedRequestDTO> decentralizedRequestDTOList, DLPUser dlpUser, DecentralizedRequest decentralizedRequest) {

        decentralizedRequest.setDecentralizedRequestId(IdWorker.get32UUID().toUpperCase());

        decentralizedRequest.setApplicantId(dlpUser.getId());

        decentralizedRequest.setDateOfApplication(LocalDateTime.now());

        decentralizedRequest.setStatus(0);

        List<DecentralizeDetails> list = new ArrayList<>();
        //接受分散采购DTO，将其赋值给分散采购明细
        for (DecentralizedRequestDTO decentralizedRequestDTO : decentralizedRequestDTOList) {

            DecentralizeDetails decentralizedDetails = new DecentralizeDetails();

            BeanUtils.copyProperties(decentralizedRequestDTO, decentralizedDetails);

            decentralizedDetails.setDecentralizeDetailsId(IdWorker.get32UUID().toUpperCase());

            decentralizedDetails.setDecentralizedRequestId(decentralizedRequest.getDecentralizedRequestId());

            list.add(decentralizedDetails);
        }
        return list;
    }

    @Override//修改分散采购申请明细
    public DecentralizeDetails editRequest(DecentralizedRequestDTO decentralizedRequestDTO) {

        DecentralizeDetails decentralizeDetails = decentralizeDetailsService.getById(decentralizedRequestDTO.getDecentralizeDetailsId());

        BeanUtils.copyProperties(decentralizedRequestDTO, decentralizeDetails);

        return decentralizeDetails;

    }

    @Override//删除分散采购申请
    public List<DecentralizeDetails> delRequestById(String decentralizedRequestId) {

        LambdaQueryWrapper<DecentralizeDetails> decentralizeDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        decentralizeDetailsLambdaQueryWrapper.eq(DecentralizeDetails::getDecentralizedRequestId, decentralizedRequestId);

        List<DecentralizeDetails> list = decentralizeDetailsService.list(decentralizeDetailsLambdaQueryWrapper);

        return list;
    }

    @Override//提交分散采购申请
    public DecentralizedRequest commitRequest(String decentralizedRequestId) {

        DecentralizedRequest decentralizedRequest = decentralizedRequestService.getById(decentralizedRequestId);

        decentralizedRequest.setStatus(1);

        return decentralizedRequest;
    }

    @Override//一级审核
    public DecentralizedRequest primaryAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDto.getUuId());

        byId.setPrimaryAuditorId(dlpUser.getId());

        byId.setAuditResultOfPrimary(auditAndApproveDto.getAuditResult());

        byId.setAuditOpinionOfPrimary(auditAndApproveDto.getAuditOpinion());

        byId.setAuditTimeOfPrimary(LocalDateTime.now());

        return byId;


    }

    @Override//二级审核
    public DecentralizedRequest secondaryAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDto.getUuId());

        byId.setSecondaryAuditorId(dlpUser.getId());

        byId.setAuditResultOfSecondary(auditAndApproveDto.getAuditResult());

        byId.setAuditOpinionOfSecondary(auditAndApproveDto.getAuditOpinion());

        byId.setAuditTimeOfSecondary(LocalDateTime.now());

        return byId;
    }

    @Override//三级审核
    public DecentralizedRequest threeLevelAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDto.getUuId());

        byId.setThreeLevelAuditId(dlpUser.getId());

        byId.setAuditResultOfThreeLevel(auditAndApproveDto.getAuditResult());

        byId.setAuditOpinionOfThreeLevel(auditAndApproveDto.getAuditOpinion());

        byId.setAuditTimeOfThreeLevel(LocalDateTime.now());

        return byId;
    }

    @Override//审批
    public DecentralizedRequest approveRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDto.getUuId());

        byId.setApproverId(dlpUser.getId());

        byId.setResultOfApproval(auditAndApproveDto.getApproveResult());

        byId.setOpinionOfApproval(auditAndApproveDto.getApproveOpinion());

        byId.setApprovalOfTime(LocalDateTime.now());

        return byId;

    }
}




