package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PurchaseCatalogueDTO;
import digital.laboratory.platform.reagent.entity.CatalogueDetails;
import digital.laboratory.platform.reagent.entity.PurchaseCatalogue;
import digital.laboratory.platform.reagent.mapper.PurchaseCatalogueMapper;
import digital.laboratory.platform.reagent.service.CatalogueDetailsService;
import digital.laboratory.platform.reagent.service.PurchaseCatalogueService;
import digital.laboratory.platform.reagent.vo.CatalogueDetailsVO;
import digital.laboratory.platform.reagent.vo.PurchaseCatalogueVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (采购目录)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购目录) 服务实现类
 */
@Service


public class PurchaseCatalogueServiceImpl extends ServiceImpl<PurchaseCatalogueMapper, PurchaseCatalogue> implements PurchaseCatalogueService {
    @Autowired
    private PurchaseCatalogueService purchaseCatalogueService;
    @Autowired
    private CatalogueDetailsService catalogueDetailsService;

    @Override
    public List<CatalogueDetails> addCatalogue(DLPUser dlpUser, List<PurchaseCatalogueDTO> purchaseCatalogueDTOList, PurchaseCatalogue purchaseCatalogue) {

        purchaseCatalogue.setPurchaseCatalogueId(IdWorker.get32UUID().toUpperCase());

        purchaseCatalogue.setStatus(0);

        List<CatalogueDetails> catalogueDetailsList = new ArrayList<>();

        for (PurchaseCatalogueDTO purchaseCatalogueDto : purchaseCatalogueDTOList) {

            CatalogueDetails catalogueDetails = new CatalogueDetails();

            BeanUtils.copyProperties(purchaseCatalogueDto, catalogueDetails);

            catalogueDetails.setCatalogueDetailsId(IdWorker.get32UUID().toUpperCase());

            catalogueDetails.setPurchaseCatalogueId(purchaseCatalogue.getPurchaseCatalogueId());

            catalogueDetailsList.add(catalogueDetails);
        }
        return catalogueDetailsList;
    }


    @Override
    public CatalogueDetails editCatalogue(PurchaseCatalogueDTO purchaseCatalogueDto) {

        CatalogueDetails catalogueDetails = catalogueDetailsService.getById(purchaseCatalogueDto.getCatalogueDetailsId());

        BeanUtils.copyProperties(purchaseCatalogueDto, catalogueDetails);

        return catalogueDetails;
    }

    @Override
    public List<CatalogueDetails> delCatalogue(String purchaseCatalogueId) {

        LambdaQueryWrapper<CatalogueDetails> catalogueDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        catalogueDetailsLambdaQueryWrapper.eq(CatalogueDetails::getPurchaseCatalogueId, purchaseCatalogueId);

        List<CatalogueDetails> list = catalogueDetailsService.list(catalogueDetailsLambdaQueryWrapper);

        if (list != null) {

            return list;
        } else {
            return null;
        }
    }

    @Override
    public IPage<PurchaseCatalogueVO> getPurchaseCatalogueVOPage(IPage<PurchaseCatalogue> page, QueryWrapper<PurchaseCatalogue> qw) {

        IPage<PurchaseCatalogueVO> purchaseCatalogueVOPage = baseMapper.getPurchaseCatalogueVOPage(page, qw);

        return purchaseCatalogueVOPage;
    }

    @Override
    public List<PurchaseCatalogueVO> getPurchaseCatalogueVOList(QueryWrapper<PurchaseCatalogue> qw) {

        List<PurchaseCatalogueVO> purchaseCatalogueVOList = baseMapper.getPurchaseCatalogueVOList(qw);

        return purchaseCatalogueVOList;
    }

    @Override
    public PurchaseCatalogueVO getPurchaseCatalogueVO(String purchaseCatalogueId) {

        PurchaseCatalogueVO purchaseCatalogueVO = baseMapper.getPurchaseCatalogueVO(purchaseCatalogueId);

        List<CatalogueDetailsVO> catalogueDetailsVOList = catalogueDetailsService.getCatalogueDetailsVOList(purchaseCatalogueId);

        purchaseCatalogueVO.setCatalogueDetailsVOListList(catalogueDetailsVOList);

        return purchaseCatalogueVO;
    }

    @Override
    public PurchaseCatalogue commitById(String purchaseCatalogueId) {

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.getById(purchaseCatalogueId);

        LambdaQueryWrapper<CatalogueDetails> catalogueDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        catalogueDetailsLambdaQueryWrapper.eq(CatalogueDetails::getPurchaseCatalogueId, purchaseCatalogueId);

        List<CatalogueDetails> list = catalogueDetailsService.list(catalogueDetailsLambdaQueryWrapper);
        //非空校验
        for (CatalogueDetails catalogueDetails : list) {

            if (catalogueDetails.getPurchaseCatalogueNumber() != null &
                    catalogueDetails.getBrand() != null &
                    catalogueDetails.getCategory() != null &
                    catalogueDetails.getNumber() != null &
                    catalogueDetails.getPackagedCopies() != null &
                    catalogueDetails.getSpecies() != null &
                    catalogueDetails.getSpecificationAndModel() != null &
                    catalogueDetails.getUnitPrice() != null &
                    catalogueDetails.getStandardValueOrPurity() != null) {
            } else {
                return null;
            }
        }
        purchaseCatalogue.setStatus(1);

        return purchaseCatalogue;
    }

    @Override
    public PurchaseCatalogue primaryAuditCatalogue(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.getById(auditAndApproveDto.getUuId());

        if (purchaseCatalogue != null) {

            purchaseCatalogue.setAuditOpinionOfPrimary(auditAndApproveDto.getAuditOpinion());

            purchaseCatalogue.setAuditResultOfPrimary(auditAndApproveDto.getAuditResult());

            purchaseCatalogue.setPrimaryAuditorId(dlpUser.getId());

            purchaseCatalogue.setAuditTimeOfPrimary(LocalDateTime.now());

            return purchaseCatalogue;
        } else {
            return null;
        }
    }

    @Override
    public PurchaseCatalogue secondaryAuditCatalogue(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.getById(auditAndApproveDto.getUuId());

        if (purchaseCatalogue != null) {

            purchaseCatalogue.setAuditOpinionOfSecondary(auditAndApproveDto.getAuditOpinion());

            purchaseCatalogue.setAuditResultOfSecondary(auditAndApproveDto.getAuditResult());

            purchaseCatalogue.setSecondaryAuditorId(dlpUser.getId());

            purchaseCatalogue.setAuditTimeOfSecondary(LocalDateTime.now());

            purchaseCatalogue.setStatus(2);

            return purchaseCatalogue;
        } else {
            return null;
        }
    }

    @Override
    public PurchaseCatalogue releaseCatalogue(String purchaseCatalogueId) {

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.getById(purchaseCatalogueId);

        if (purchaseCatalogue != null) {

            purchaseCatalogue.setStatus(3);

            return purchaseCatalogue;
        } else {
            return null;
        }
    }
}
