package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.dto.PurchaseListDTO;
import digital.laboratory.platform.reagent.entity.PurchaseList;
import digital.laboratory.platform.reagent.entity.PurchaseListDetails;
import digital.laboratory.platform.reagent.mapper.PurchaseListMapper;
import digital.laboratory.platform.reagent.service.PurchaseListDetailsService;
import digital.laboratory.platform.reagent.service.PurchaseListService;
import digital.laboratory.platform.reagent.vo.PurchaseListVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * (采购清单)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购清单) 服务实现类
 */
@Service
public class PurchaseListServiceImpl extends ServiceImpl<PurchaseListMapper, PurchaseList> implements PurchaseListService {

    @Autowired
    private PurchaseListService purchaseListService;
    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Override//通过ID查找清单
    public PurchaseListVO getPurchaseList(String purchaseListId) {

        PurchaseList byId = purchaseListService.getById(purchaseListId);

        List<PurchaseListDetails> list = purchaseListDetailsService.getPurchaseListDetailsVOList(purchaseListId);

        PurchaseListVO purchaseListVO = new PurchaseListVO();

        BeanUtils.copyProperties(byId, purchaseListVO);

        purchaseListVO.setPurchaseListDetailsList(list);

        return purchaseListVO;
    }

    @Transactional
    @Override//新增采购清单
    public PurchaseList addListById(List<PurchaseListDTO> purchaseListDTOList, PurchaseList purchaseList){

        purchaseList.setPurchaseListId(IdWorker.get32UUID().toUpperCase());

        List<PurchaseListDetails> purchaseListDetailsList= new ArrayList<>();

        for (PurchaseListDTO purchaseListDTO : purchaseListDTOList) {

            PurchaseListDetails purchaseListDetails = new PurchaseListDetails();

            BeanUtils.copyProperties(purchaseListDTO,purchaseListDetails);

            purchaseListDetails.setPurchaseListId(purchaseList.getPurchaseListId());

            purchaseListDetails.setPurchaselistDetailsId(IdWorker.get32UUID().toUpperCase());

            purchaseListDetailsList.add(purchaseListDetails);
        }
        if (purchaseListService.save(purchaseList)&purchaseListDetailsService.saveBatch(purchaseListDetailsList)){
            return purchaseList;
        }else {
            return null;
        }

    }

}
