package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PurchasingPlanDTO;
import digital.laboratory.platform.reagent.entity.ProcurementContent;
import digital.laboratory.platform.reagent.entity.PurchasingPlan;
import digital.laboratory.platform.reagent.mapper.PurchasingPlanMapper;
import digital.laboratory.platform.reagent.service.ProcurementContentService;
import digital.laboratory.platform.reagent.service.PurchasingPlanService;
import digital.laboratory.platform.reagent.vo.ProcurementContentVO;
import digital.laboratory.platform.reagent.vo.PurchasingPlanVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (采购计划)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购计划) 服务实现类
 */
@Service
public class PurchasingPlanServiceImpl extends ServiceImpl<PurchasingPlanMapper, PurchasingPlan> implements PurchasingPlanService {

    @Autowired
    private PurchasingPlanService purchasingPlanService;
    @Autowired
    private ProcurementContentService procurementContentService;

    @Override//分页
    public IPage<PurchasingPlanVO> getPurchasingPlanVOPage(IPage<PurchasingPlan> page, QueryWrapper<PurchasingPlan> qw) {
        IPage<PurchasingPlanVO> purchasingPlanVOPage = baseMapper.getPurchasingPlanVOPage(page, qw);
        return purchasingPlanVOPage;
    }

    @Override//列表
    public List<PurchasingPlanVO> getPurchasingPlanVOList(QueryWrapper<PurchasingPlan> qw) {
        return baseMapper.getPurchasingPlanVOList(qw);
    }

    @Override//通过ID查询
    public PurchasingPlanVO getPurchasingPlanVO(String purchasingPlanId) {

        PurchasingPlanVO purchasingPlanVO = baseMapper.getPurchasingPlanVO(purchasingPlanId);

        if (purchasingPlanVO == null) {

            throw new RuntimeException(String.format("信息不存在"));
        }

        List<ProcurementContentVO> procurementContentVOList = procurementContentService.getProcurementContentVOList(purchasingPlanId);

        purchasingPlanVO.setProcurementContentVOList(procurementContentVOList);

        return purchasingPlanVO;
    }

    @Transactional
    @Override//创建计划
    public Boolean addById(List<PurchasingPlanDTO> purchasingPlanDTOList, DLPUser user, PurchasingPlan purchasingPlan) {

        purchasingPlan.setPurchasingPlanId(IdWorker.get32UUID().toUpperCase());

        purchasingPlan.setCreateId(user.getId());

        purchasingPlan.setStatus(0);

        List<ProcurementContent> list = new ArrayList<>();

        for (PurchasingPlanDTO purchasingPlanDTO : purchasingPlanDTOList) {

            ProcurementContent procurementContent = new ProcurementContent();

            BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);

            procurementContent.setPurchasingPlanId(purchasingPlan.getPurchasingPlanId());

            procurementContent.setProcurementContentId(IdWorker.get32UUID().toUpperCase());

            list.add(procurementContent);
        }

        return purchasingPlanService.save(purchasingPlan) & procurementContentService.saveBatch(list);
    }

    @Override
    public ProcurementContent addContent(PurchasingPlanDTO purchasingPlanDTO) {

        ProcurementContent procurementContent = new ProcurementContent();

        BeanUtils.copyProperties(purchasingPlanDTO, procurementContent);

        if (procurementContentService.save(procurementContent)) {
            return procurementContent;
        } else {
            return null;
        }
    }

    @Override
    public ProcurementContent editById(PurchasingPlanDTO purchasingPlanDTO) {

        ProcurementContent byId = procurementContentService.getById(purchasingPlanDTO.getProcurementContentId());

        BeanUtils.copyProperties(purchasingPlanDTO, byId);

        if (purchasingPlanService.getById(byId.getPurchasingPlanId()).getStatus() == 0 & procurementContentService.updateById(byId)) {
            return byId;
        } else {
            return null;
        }


    }

    @Override
    @Transactional
    public Boolean delById(String purchasingPlanId) {

        PurchasingPlan purchasingPlan = purchasingPlanService.getById(purchasingPlanId);

        LambdaQueryWrapper<ProcurementContent> procurementContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

        procurementContentLambdaQueryWrapper.eq(ProcurementContent::getPurchasingPlanId, purchasingPlan.getPurchasingPlanId());

        List<ProcurementContent> list = procurementContentService.list(procurementContentLambdaQueryWrapper);

        if (purchasingPlan.getStatus() == 0) {
            return procurementContentService.removeBatchByIds(list) & purchasingPlanService.removeById(purchasingPlanId);
        } else {
            return false;
        }
    }

    @Override
    public Boolean delContentById(String procurementContentId) {

        ProcurementContent byId = procurementContentService.getById(procurementContentId);

        if (purchasingPlanService.getById(byId.getPurchasingPlanId()).getStatus() == 0) {
            return procurementContentService.removeById(procurementContentId);
        } else {
            return null;
        }
    }

    @Override
    public PurchasingPlan commitById(String purchasingPlanId) {

        PurchasingPlan byId = purchasingPlanService.getById(purchasingPlanId);

        LambdaQueryWrapper<ProcurementContent> procurementContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

        procurementContentLambdaQueryWrapper.eq(ProcurementContent::getPurchasingPlanId, byId.getPurchasingPlanId());

        List<ProcurementContent> list = procurementContentService.list(procurementContentLambdaQueryWrapper);

        for (ProcurementContent procurementContent : list) {

            if (!(procurementContent.getUnitPrice() > 0 &
                    procurementContent.getUnitPrice() < 1000000 &
                    procurementContent.getQuantity() > 0 &
                    procurementContent.getQuantity() < 1000 &
                    procurementContent.getReagentConsumableId() != null)) {
                throw new RuntimeException(String.format("您输入的值有误，请重新输入"));
            }
        }
        byId.setStatus(1);
        if (purchasingPlanService.updateById(byId)) {
            return byId;
        } else {
            return null;
        }
    }

    @Override
    public PurchasingPlan auditById(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchasingPlan purchasingPlan = purchasingPlanService.getById(auditAndApproveDto.getUuId());

        if (purchasingPlan.getStatus() != 1) {
            throw new RuntimeException(String.format("当前状态不可审核"));
        }
        purchasingPlan.setAuditResultOfPrimary(auditAndApproveDto.getAuditResult());

        purchasingPlan.setAuditOpinionOfPrimary(auditAndApproveDto.getAuditOpinion());

        purchasingPlan.setAuditTimeOfPrimary(LocalDateTime.now());

        purchasingPlan.setPrimaryAuditorId(dlpUser.getId());

        purchasingPlan.setStatus(2);

        if (purchasingPlanService.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else {
            return null;
        }
    }

    @Override
    public PurchasingPlan approveById(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {

        PurchasingPlan purchasingPlan = purchasingPlanService.getById(auditAndApproveDto.getUuId());

        if (purchasingPlan.getStatus() != 2) {
            throw new RuntimeException(String.format("当前状态不可审批"));
        }
        purchasingPlan.setApprovalResult(auditAndApproveDto.getApproveResult());

        purchasingPlan.setApprovalOpinion(auditAndApproveDto.getApproveOpinion());

        purchasingPlan.setApprovalTime(LocalDateTime.now());

        purchasingPlan.setStatus(3);

        purchasingPlan.setApproverId(dlpUser.getId());

        if (purchasingPlanService.updateById(purchasingPlan)) {
            return purchasingPlan;
        } else {
            return null;
        }
    }

    @Override
    public PurchasingPlan releaseById(String purchasingPlanId) {

        PurchasingPlan byId = purchasingPlanService.getById(purchasingPlanId);

        if (byId.getStatus() != 3) {
            throw new RuntimeException(String.format("当前状态无法提交"));
        }
        byId.setStatus(4);

        if (purchasingPlanService.updateById(byId)) {
            return byId;
        } else {
            return null;
        }
    }
}
